==> labbench-0.22.tar.gz: labbench-0.22/tests/test_property.py <==
# This software was developed by employees of the National Institute of
# Standards and Technology (NIST), an agency of the Federal Government.
# Pursuant to title 17 United Property Code Section 105, works of NIST employees
# are not subject to copyright protection in the United Property and are
# considered to be in the public domain. Permission to freely use, copy,
# modify, and distribute this software and its documentation without fee is
# hereby granted, provided that this notice and disclaimer of warranty appears
# in all copies.
#
# THE SOFTWARE IS PROVIDED 'AS IS' WITHOUT ANY WARRANTY OF ANY KIND, EITHER
# EXPRESSED, IMPLIED, OR STATUTORY, INCLUDING, BUT NOT LIMITED TO, ANY WARRANTY
# THAT THE SOFTWARE WILL CONFORM TO SPECIFICATIONS, ANY IMPLIED WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, AND FREEDOM FROM
# INFRINGEMENT, AND ANY WARRANTY THAT THE DOCUMENTATION WILL CONFORM TO THE
# SOFTWARE, OR ANY WARRANTY THAT THE SOFTWARE WILL BE ERROR FREE. IN NO EVENT
# SHALL NIST BE LIABLE FOR ANY DAMAGES, INCLUDING, BUT NOT LIMITED TO, DIRECT,
# INDIRECT, SPECIAL OR CONSEQUENTIAL DAMAGES, ARISING OUT OF, RESULTING FROM,
# OR IN ANY WAY CONNECTED WITH THIS SOFTWARE, WHETHER OR NOT BASED UPON
# WARRANTY, CONTRACT, TORT, OR OTHERWISE, WHETHER OR NOT INJURY WAS SUSTAINED
# BY PERSONS OR Decorator OR OTHERWISE, AND WHETHER OR NOT LOSS WAS SUSTAINED
# FROM, OR AROSE OUT OF THE RESULTS OF, OR USE OF, THE SOFTWARE OR SERVICES
# PROVIDED HEREUNDER. Distributions of NIST software should also include
# copyright and licensing statements of any third-party software that are
# legally bundled with the code in compliance with the conditions of those
# licenses.

import unittest
import importlib
import sys
if '..' not in sys.path:
    sys.path.insert(0, '..')
import labbench as lb
from copy import copy
lb._force_full_traceback(True)


class TestProperty:
    TestDevice = None

    def test_get_set(self):
        with self.TestDevice() as m:
            m.clear_counts()

            # remapped bool "get"
            self.assertEqual(m.int0, m.PYTHONIC_VALUE_DEFAULT['int0'])

            # "set"
            new_param = m.int0 + 1
            m.int0 = new_param
            self.assertEqual(m.int0, new_param)
            self.assertEqual(m.remote_values['int0'], new_param)

            # verify expected "get"
            self.assertEqual(m.bool0,
                             m.PYTHONIC_VALUE_DEFAULT['bool0'])  # get 1

            # verify expected "set"
            new_flag = not m.bool0  # get 2
            m.bool0 = new_flag

            self.assertEqual(m.bool0, new_flag)  # get 3

            self.assertEqual(m.remote_values['bool0'],
                             m.get_expected_remote_value('bool0', new_flag))

            self.assertEqual(m._getter_counts['bool0'], 3)
            self.assertEqual(m._getter_counts['int0'], 3)


class MockBase(lb.Device):
    # dictionary of mapping dictionaries, keyed on class attribute name
    REMAP = {}

    # the default value for bool traits {trait_name: trait_value}
    PYTHONIC_VALUE_DEFAULT = {}

    # test values for setting the property traits {trait_name: trait_value}
    PYTHONIC_VALUE_UPDATE = {}

    _getter_counts = {}

    def open(self):
        self._getter_counts = {}
        self.remote_values = {}
        self._last = {}

        for name, value in self.PYTHONIC_VALUE_DEFAULT.items():
            self.remote_values[name] = self.get_expected_remote_value(
                name, value)

    def add_get_count(self, name):
        self._getter_counts.setdefault(name, 0)
        self._getter_counts[name] += 1

    def clear_counts(self):
        self._getter_counts = {}

    def get_get_count(self, name):
        return self._getter_counts.get(name, 0)

    def get_expected_remote_value(self, name, value):
        return self.REMAP.get(name, {}).get(value, value)


class MockDecoratedProperty(MockBase):
    PYTHONIC_VALUE_DEFAULT = {
        'int0': 3,
        'bool0': False,
        'str0': 'moose',
        'str1': 'moose',
        'str2': 'moose'
    }

    PYTHONIC_VALUE_UPDATE = {
        'int0': 4,
        'bool0': True,
        'str0': 'hi',
        'str1': 'hi',
        'str2': 'hi'
    }

    REMAP = dict(bool0={True: 'ON', False: 'OFF'})

    @lb.property.int(min=0, max=10)
    def int0(self, value):
        self.remote_values['int0'] = value

    def int0(self):
        self.add_get_count('int0')
        return self.remote_values['int0']

    @lb.property.bool(remap=REMAP['bool0'])
    def bool0(self, value):
        self.remote_values['bool0'] = value

    def bool0(self):
        self.add_get_count('bool0')
        return self.remote_values['bool0']

    @lb.property.bool(remap=REMAP['bool0'])
    def bool0(self, value):
        self.remote_values['bool0'] = value

    def bool0(self):
        self.add_get_count('bool0')
        return self.remote_values['bool0']

    @lb.property.str(key='str0', cache=True)
    def str0(self, value):
        self.remote_values['str0'] = value

    def str0(self):
        self.add_get_count('str0')
        return self.remote_values['str0']

    @lb.property.str(key='str1',
                     cache=True,
                     remap={'python value': 'device value'})
    def str1(self, value):
        self.remote_values['str1'] = value

    def str1(self):
        self.add_get_count('str1')
        return self.remote_values['str1']

    @lb.property.str(key='str2', sets=False)
    def str2(self, value):
        self.remote_values['str2'] = value

    def str2(self):
        self.add_get_count('str2')
        return self.remote_values['str2']


class MockKeyedProperty(MockBase):
    # configure the emulated behavior of the fake instrument
    PYTHONIC_VALUE_DEFAULT = {
        'int0': 3,
        'bool0': False,
        'str0': 'moose',
        'str1': 'moose',
        'str2': 'moose'
    }

    PYTHONIC_VALUE_UPDATE = {
        'int0': 4,
        'bool0': True,
        'str0': 'hi',
        'str1': 'hi',
        'str2': 'hi'
    }

    REMAP = dict(bool0={True: 'ON', False: 'OFF'})

    int0 = lb.property.int(key='int0', min=0, max=10)
    bool0 = lb.property.bool(key='bool0', remap=REMAP['bool0'])
    str0 = lb.property.str(key='str0', cache=True)
    str1 = lb.property.str(key='str1',
                           cache=True,
                           remap={'python value': 'device value'})
    str2 = lb.property.str(key='str2', sets=False)

    def get_key(self, key, name=None):
        print('get ', name, key)
        self.add_get_count(key)
        return self.remote_values[key]

    def set_key(self, key, value, name=None):
        self._last[key] = value
        self.remote_values[key] = self.get_expected_remote_value(name, value)


class MockReturner(MockBase):

    @lb.datareturn.float()
    def fetch_float0(self, a, b, c):
        return a + b + c

    @lb.datareturn.float
    def fetch_float1(self, a, b, c):
        return a + b + c


class TestProperty:
    # set this in a subclass
    TestDevice = None

    def test_set_then_get(self):
        with self.TestDevice() as m:
            m.clear_counts()

            for trait_name, new_value in m.PYTHONIC_VALUE_UPDATE.items():
                msg = f'property "{trait_name}"'
                trait = m._traits[trait_name]

                if not (trait.sets and trait.gets):
                    # this test is only for traits that support both set and get
                    continue

                m.clear_counts()
                setattr(m, trait_name, new_value)

                # in cases of remap, ensure the stored value in the mock device
                # matches the expected
                self.assertEqual(m.remote_values[trait_name],
                                 m.get_expected_remote_value(
                                     trait_name, new_value),
                                 msg=msg)

                # validate the pythonic value
                self.assertEqual(getattr(m, trait_name), new_value,
                                 msg=msg)  # +1 get (unless cached)

                # make sure there weren't any unecessary extra 'get' operations
                self.assertEqual(m.get_get_count(trait_name),
                                 0 if trait.cache else 1,
                                 msg=msg)

    def test_disabled_set(self):
        with self.TestDevice() as m:
            m.clear_counts()

            for trait_name, new_value in m.PYTHONIC_VALUE_UPDATE.items():
                trait = m._traits[trait_name]
                msg = f'property "{trait_name}"'

                if not trait.sets:
                    with self.assertRaises(AttributeError, msg=msg):
                        setattr(m, trait_name, new_value)

    def test_cache(self):
        with self.TestDevice() as m:
            m.clear_counts()

            for trait_name, new_value in m.PYTHONIC_VALUE_UPDATE.items():
                if m._traits[trait_name].cache:
                    msg = f'property "{trait_name}"'

                    # Device gets should only happen once if cache=True
                    m.clear_counts()
                    self.assertEqual(getattr(m, trait_name),
                                     m.PYTHONIC_VALUE_DEFAULT[trait_name],
                                     msg=msg)
                    self.assertEqual(getattr(m, trait_name),
                                     m.PYTHONIC_VALUE_DEFAULT[trait_name],
                                     msg=msg)
                    self.assertEqual(m.get_get_count(trait_name), 1, msg=msg)

    def test_get_default(self):
        """
        Ensure that values are returned, typing is correct, and 
        """
        with self.TestDevice() as m:
            m.clear_counts()

            # remapped bool "get"
            for trait_name, value in m.PYTHONIC_VALUE_DEFAULT.items():
                msg = f'testing property "{trait_name}"'
                self.assertEqual(getattr(m, trait_name), value, msg=msg)
                self.assertEqual(m.get_get_count(trait_name), 1, msg=msg)


class TestDecoratedProperty(unittest.TestCase, TestProperty):
    TestDevice = MockDecoratedProperty


class TestKeyedProperty(unittest.TestCase, TestProperty):
    TestDevice = MockDecoratedProperty


class TestReturner(unittest.TestCase):

    def test_returner_type(self):
        with MockReturner() as m:
            self.assertEqual(m.fetch_float0(1, 2, 3), 6.)
            self.assertEqual(m.fetch_float1(1, 2, 3), 6.)


if __name__ == '__main__':
    lb.show_messages('debug')
    unittest.main()

    # with MockDecoratedProperty() as m:
    #     pass

==> labbench-0.25.0.tar.gz: labbench-0.25.0/tests/test_property.py <==
# This software was developed by employees of the National Institute of
# Standards and Technology (NIST), an agency of the Federal Government.
# Pursuant to title 17 United Property Code Section 105, works of NIST employees
# are not subject to copyright protection in the United Property and are
# considered to be in the public domain. Permission to freely use, copy,
# modify, and distribute this software and its documentation without fee is
# hereby granted, provided that this notice and disclaimer of warranty appears
# in all copies.
#
# THE SOFTWARE IS PROVIDED 'AS IS' WITHOUT ANY WARRANTY OF ANY KIND, EITHER
# EXPRESSED, IMPLIED, OR STATUTORY, INCLUDING, BUT NOT LIMITED TO, ANY WARRANTY
# THAT THE SOFTWARE WILL CONFORM TO SPECIFICATIONS, ANY IMPLIED WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, AND FREEDOM FROM
# INFRINGEMENT, AND ANY WARRANTY THAT THE DOCUMENTATION WILL CONFORM TO THE
# SOFTWARE, OR ANY WARRANTY THAT THE SOFTWARE WILL BE ERROR FREE. IN NO EVENT
# SHALL NIST BE LIABLE FOR ANY DAMAGES, INCLUDING, BUT NOT LIMITED TO, DIRECT,
# INDIRECT, SPECIAL OR CONSEQUENTIAL DAMAGES, ARISING OUT OF, RESULTING FROM,
# OR IN ANY WAY CONNECTED WITH THIS SOFTWARE, WHETHER OR NOT BASED UPON
# WARRANTY, CONTRACT, TORT, OR OTHERWISE, WHETHER OR NOT INJURY WAS SUSTAINED
# BY PERSONS OR Decorator OR OTHERWISE, AND WHETHER OR NOT LOSS WAS SUSTAINED
# FROM, OR AROSE OUT OF THE RESULTS OF, OR USE OF, THE SOFTWARE OR SERVICES
# PROVIDED HEREUNDER. Distributions of NIST software should also include
# copyright and licensing statements of any third-party software that are
# legally bundled with the code in compliance with the conditions of those
# licenses.

import unittest
import importlib
import sys

if ".." not in sys.path:
    sys.path.insert(0, "..")
import labbench as lb
from copy import copy

lb._force_full_traceback(True)


class TestProperty:
    TestDevice = None

    def test_get_set(self):
        with self.TestDevice() as m:
            m.clear_counts()

            # remapped bool "get"
            self.assertEqual(m.int0, m.PYTHONIC_VALUE_DEFAULT["int0"])

            # "set"
            new_param = m.int0 + 1
            m.int0 = new_param
            self.assertEqual(m.int0, new_param)
            self.assertEqual(m.remote_values["int0"], new_param)

            # verify expected "get"
            self.assertEqual(m.bool0, m.PYTHONIC_VALUE_DEFAULT["bool0"])  # get 1

            # verify expected "set"
            new_flag = not m.bool0  # get 2
            m.bool0 = new_flag

            self.assertEqual(m.bool0, new_flag)  # get 3

            self.assertEqual(
                m.remote_values["bool0"], m.get_expected_remote_value("bool0", new_flag)
            )

            self.assertEqual(m._getter_counts["bool0"], 3)
            self.assertEqual(m._getter_counts["int0"], 3)


class MockBase(lb.Device):
    # dictionary of mapping dictionaries, keyed on class attribute name
    REMAP = {}

    # the default value for bool traits {trait_name: trait_value}
    PYTHONIC_VALUE_DEFAULT = {}

    # test values for setting the property traits {trait_name: trait_value}
    PYTHONIC_VALUE_UPDATE = {}

    _getter_counts = {}

    def open(self):
        self._getter_counts = {}
        self.remote_values = {}
        self._last = {}

        for name, value in self.PYTHONIC_VALUE_DEFAULT.items():
            self.remote_values[name] = self.get_expected_remote_value(name, value)

    def add_get_count(self, name):
        self._getter_counts.setdefault(name, 0)
        self._getter_counts[name] += 1

    def clear_counts(self):
        self._getter_counts = {}

    def get_get_count(self, name):
        return self._getter_counts.get(name, 0)

    def get_expected_remote_value(self, name, value):
        return self.REMAP.get(name, {}).get(value, value)


class MockDecoratedProperty(MockBase):
    PYTHONIC_VALUE_DEFAULT = {
        "int0": 3,
        "bool0": False,
        "str0": "moose",
        "str1": "moose",
        "str2": "moose",
    }

    PYTHONIC_VALUE_UPDATE = {
        "int0": 4,
        "bool0": True,
        "str0": "hi",
        "str1": "hi",
        "str2": "hi",
    }

    REMAP = dict(bool0={True: "ON", False: "OFF"})

    @lb.property.int(min=0, max=10)
    def int0(self, value):
        self.remote_values["int0"] = value

    def int0(self):
        self.add_get_count("int0")
        return self.remote_values["int0"]

    @lb.property.bool(remap=REMAP["bool0"])
    def bool0(self, value):
        self.remote_values["bool0"] = value

    def bool0(self):
        self.add_get_count("bool0")
        return self.remote_values["bool0"]

    @lb.property.bool(remap=REMAP["bool0"])
    def bool0(self, value):
        self.remote_values["bool0"] = value

    def bool0(self):
        self.add_get_count("bool0")
        return self.remote_values["bool0"]

    @lb.property.str(key="str0", cache=True)
    def str0(self, value):
        self.remote_values["str0"] = value

    def str0(self):
        self.add_get_count("str0")
        return self.remote_values["str0"]

    @lb.property.str(key="str1", cache=True, remap={"python value": "device value"})
    def str1(self, value):
        self.remote_values["str1"] = value

    def str1(self):
        self.add_get_count("str1")
        return self.remote_values["str1"]

    @lb.property.str(key="str2", sets=False)
    def str2(self, value):
        self.remote_values["str2"] = value

    def str2(self):
        self.add_get_count("str2")
        return self.remote_values["str2"]


class MockKeyedProperty(MockBase):
    # configure the emulated behavior of the fake instrument
    PYTHONIC_VALUE_DEFAULT = {
        "int0": 3,
        "bool0": False,
        "str0": "moose",
        "str1": "moose",
        "str2": "moose",
    }

    PYTHONIC_VALUE_UPDATE = {
        "int0": 4,
        "bool0": True,
        "str0": "hi",
        "str1": "hi",
        "str2": "hi",
    }

    REMAP = dict(bool0={True: "ON", False: "OFF"})

    int0 = lb.property.int(key="int0", min=0, max=10)
    bool0 = lb.property.bool(key="bool0", remap=REMAP["bool0"])
    str0 = lb.property.str(key="str0", cache=True)
    str1 = lb.property.str(
        key="str1", cache=True, remap={"python value": "device value"}
    )
    str2 = lb.property.str(key="str2", sets=False)

    def get_key(self, key, name=None):
        print("get ", name, key)
        self.add_get_count(key)
        return self.remote_values[key]

    def set_key(self, key, value, name=None):
        self._last[key] = value
        self.remote_values[key] = self.get_expected_remote_value(name, value)


class MockReturner(MockBase):
    @lb.datareturn.float()
    def fetch_float0(self, a, b, c):
        return a + b + c

    @lb.datareturn.float
    def fetch_float1(self, a, b, c):
        return a + b + c


class TestProperty:
    # set this in a subclass
    TestDevice = None

    def test_set_then_get(self):
        with self.TestDevice() as m:
            m.clear_counts()

            for trait_name, new_value in m.PYTHONIC_VALUE_UPDATE.items():
                msg = f'property "{trait_name}"'
                trait = m._traits[trait_name]

                if not (trait.sets and trait.gets):
                    # this test is only for traits that support both set and get
                    continue

                m.clear_counts()
                setattr(m, trait_name, new_value)

                # in cases of remap, ensure the stored value in the mock device
                # matches the expected
                self.assertEqual(
                    m.remote_values[trait_name],
                    m.get_expected_remote_value(trait_name, new_value),
                    msg=msg,
                )

                # validate the pythonic value
                self.assertEqual(
                    getattr(m, trait_name), new_value, msg=msg
                )  # +1 get (unless cached)

                # make sure there weren't any unecessary extra 'get' operations
                self.assertEqual(
                    m.get_get_count(trait_name), 0 if trait.cache else 1, msg=msg
                )

    def test_disabled_set(self):
        with self.TestDevice() as m:
            m.clear_counts()

            for trait_name, new_value in m.PYTHONIC_VALUE_UPDATE.items():
                trait = m._traits[trait_name]
                msg = f'property "{trait_name}"'

                if not trait.sets:
                    with self.assertRaises(AttributeError, msg=msg):
                        setattr(m, trait_name, new_value)

    def test_cache(self):
        with self.TestDevice() as m:
            m.clear_counts()

            for trait_name, new_value in m.PYTHONIC_VALUE_UPDATE.items():
                if m._traits[trait_name].cache:
                    msg = f'property "{trait_name}"'

                    # Device gets should only happen once if cache=True
                    m.clear_counts()
                    self.assertEqual(
                        getattr(m, trait_name),
                        m.PYTHONIC_VALUE_DEFAULT[trait_name],
                        msg=msg,
                    )
                    self.assertEqual(
                        getattr(m, trait_name),
                        m.PYTHONIC_VALUE_DEFAULT[trait_name],
                        msg=msg,
                    )
                    self.assertEqual(m.get_get_count(trait_name), 1, msg=msg)

    def test_get_default(self):
        """
        Ensure that values are returned, typing is correct, and
        """
        with self.TestDevice() as m:
            m.clear_counts()

            # remapped bool "get"
            for trait_name, value in m.PYTHONIC_VALUE_DEFAULT.items():
                msg = f'testing property "{trait_name}"'
                self.assertEqual(getattr(m, trait_name), value, msg=msg)
                self.assertEqual(m.get_get_count(trait_name), 1, msg=msg)


class TestDecoratedProperty(unittest.TestCase, TestProperty):
    TestDevice = MockDecoratedProperty


class TestKeyedProperty(unittest.TestCase, TestProperty):
    TestDevice = MockDecoratedProperty


class TestReturner(unittest.TestCase):
    def test_returner_type(self):
        with MockReturner() as m:
            self.assertEqual(m.fetch_float0(1, 2, 3), 6.0)
            self.assertEqual(m.fetch_float1(1, 2, 3), 6.0)


if __name__ == "__main__":
    lb.show_messages("debug")
    unittest.main()

    # with MockDecoratedProperty() as m:
    #     pass
